/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright 2023 Mellanox Technologies, Ltd
 *
 * This file contains concrete implementation of the function pinters introduced
 * in RTE generic flow perf. The implementation depends on RTE flow library,
 * this is to benchmark its performance.
 */

#ifndef _FLOW_PERF_H_
#define _FLOW_PERF_H_

#ifdef __cplusplus
extern "C" {
#endif

struct flow_perf_layer_idata {
	struct rte_flow_pattern_template *it;
	struct rte_flow_item *items;
};

struct flow_perf_layer_adata {
	struct rte_flow_action *actions;
	struct rte_flow_action *masks;
	struct rte_flow_actions_template *at;
};

struct flow_perf_layer_action_element {
	struct rte_flow_action action;
	struct rte_flow_action mask;
};

struct flow_perf_layer_data {
	struct rte_flow_item *items;
	struct rte_flow_action *actions;
	struct rte_flow_action *masks;
	struct rte_flow_template_table *table;
	/* Save background flows only. */
	struct rte_flow *flow[];
};

/* Changeable item & action conf. */
union flow_perf_layer_conf_data {
	/* Items to be added. */
	struct rte_flow_item_eth eth;
	struct rte_flow_item_ipv4 ipv4;
	struct rte_flow_item_ipv6 ipv6;
	struct rte_flow_item_meter_color mtr_mark;
	struct rte_flow_item_udp udp;
	struct rte_flow_item_tcp tcp;
	struct rte_flow_item_meta meta;
	struct rte_flow_item_gtp gtp;

	/* Actions to be added. */
	struct rte_flow_action_modify_field modify_field;
	uint8_t raw_encap[256];
	struct perf_flow_indir_action *pfia;
};

struct rte_flow_perf_pipeline *
args_parse(const char *json_file_path);

int
flow_perf_init(struct rte_flow_perf_pipeline *ppl);

int
create_pipes(struct rte_flow_perf_flow_info *fi, struct rte_flow_perf_flow_data *pfd);

int
destroy(void);

void
set_idata_priv_field_addr(struct rte_flow_perf_idata *pfit);

void
set_adata_priv_field_addr(struct rte_flow_perf_adata *pfat, uint32_t anum);

void
set_idata_priv_field_vals(struct rte_flow_perf_element *pfi,
			uint32_t nb_items, bool init_v,
			struct rte_flow_perf_idata *pfit);

void
set_adata_priv_field_vals(struct rte_flow_perf_flow_data *pfd,
			struct rte_flow_perf_element *pat, uint32_t nb_actions,
			bool init_v, uint32_t nb_flow, bool clone,
			struct rte_flow_perf_adata *pfat);
int
flow_perf_layer_add_bg_flows(struct rte_flow_perf_flow_data *pfd,
			uint32_t postpone, uint32_t i,
			uint32_t *fidx, uint32_t *miss_count);

int
flow_perf_layer_rm_bg_flows(struct rte_flow_perf_flow_data *pfd,
			uint32_t postpone, uint32_t i, uint32_t *fidx, uint32_t *miss_count);

int
_handle_dyn_flows(uint16_t port_id, uint16_t queue,
		enum rte_flow_perf_mode mode,
		struct rte_flow_perf_stat *st,
		struct rte_flow_perf_flow_data *pfd,
		struct rte_flow_perf_per_core_data *pcfd);
int
push_dyn_flows(uint32_t queue, uint32_t *fidx, struct rte_flow_perf_stat *st);

void
destroy_flow_data_priv(uint32_t port_id, void *flow_data_priv);

void
destroy_idata_priv(uint32_t port_id, void *idata_priv);

void
destroy_adata_priv(uint32_t port_id, void *adata_priv);

#ifdef __cplusplus
}
#endif

#endif /* _FLOW_PERF_H_ */
